import argparse

import feynml
from feynmodel.interface.qgraf import feynmodel_to_qgraf, qgraf_to_feynmodel
from feynmodel.interface.ufo import load_ufo_model
from pyfeyn2.auto.diagram import auto_diagram
from pyfeyn2.render.latex.tikzfeynman import TikzFeynmanRender
from pyqgraf import qgraf

from ufo_draw.ufo_diagrams import generate_diagrams


def main():
    # parse command line options with argparse
    parser = argparse.ArgumentParser(
        prog="ufo-draw.ufo_draw",
        description="Draw FeynML diagrams from ufo models with pyfeyn2.",
    )
    parser.add_argument(
        "-p",
        "-m",
        "--path",
        "--model",
        type=str,
        default="ufo_sm",
        help="Path to UFO model directory.",
    )
    parser.add_argument(
        "-l",
        "--loops",
        type=int,
        default=0,
        help="Number of loops to draw.",
    )
    parser.add_argument(
        "-o",
        "--output",
        type=str,
        default="diagram",
        help="Output file name without suffix.",
    )
    # argument list of incoming particles
    parser.add_argument(
        "-i",
        "--initial",
        type=str,
        default="e- e+",
        help="Incoming particles.",
    )
    # argument list of outgoing particles
    parser.add_argument(
        "-f",
        "--final",
        type=str,
        default="e- e+",
        help="Outgoing particles.",
    )
    parser.add_argument(
        "--fml",
        action="store_true",
        help="Output FeynML.",
        default=False,
    )
    # TODO convert general particle input to less general particle input
    # TODO filter orders
    # TODO filter propagators

    # TODO show option?

    args = parser.parse_args()

    fml = generate_diagrams(
        args.path, args.initial.split(" "), args.final.split(" "), args.loops
    )

    # Render diagrams
    for i, d in enumerate(fml.diagrams):
        auto_diagram(d)
        t = TikzFeynmanRender(d)
        t.render(show=True, file=args.output + f"_{i}")
    # Write FML
    if args.fml:
        s = fml.to_xml()
        # print s to args.output + ".fml"
        with open(args.output + ".fml", "w") as f:
            f.write(s)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ufo_draw']

package_data = \
{'': ['*']}

install_requires = \
['feynml', 'feynmodel>=0.0.4', 'pyfeyn2>=2.3.4', 'pyqgraf>=0.0.8']

entry_points = \
{'console_scripts': ['ufo_draw = ufo_draw.main:main']}

setup_kwargs = {
    'name': 'ufo-draw',
    'version': '0.0.1.19',
    'description': 'Draw Feynman diagrams from UFO models',
    'long_description': '# ufo_draw\n\nAllows you to quickly draw feynman diagrams from ufo model files.\n\n## Installation\n\n```bash\npip install ufo_draw # Install it\n```\nwith optional `--user` or `--break-system-packages`. \nHowever due to the amount of dependencies it might be better to install it using pipx to have dependencies in isolated versions\n\n```bash\npipx install ufo_draw\n```\n\n## Example\n\n```\nufo_draw --initial "nu_e nu_e_bar" --final "nu_e nu_e_bar" -o diagram -m ufo_sm\n```\ncreates following diagrams via [pyfeyn2](https://github.com/APN-Pucky/pyfeyn2) from the [ufo_sm](https://github.com/APN-Pucky/ufo_sm) model using [pyqgraf](https://github.com/APN-Pucky/pyqgraf).\n\n![diag0](./img/diagram_0.png)\n![diag1](./img/diagram_1.png)\n',
    'author': 'Alexander Puck Neuwirth',
    'author_email': 'alexander@neuwirth-informatik.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/APN-Pucky/ufo-draw',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import List
from pyqgraf import qgraf

from feynmodel.interface.qgraf import feynmodel_to_qgraf, qgraf_to_feynmodel
from feynmodel.interface.ufo import load_ufo_model

from xsdata.formats.dataclass.parsers import XmlParser
from pyfeyn2.feynmandiagram import FeynML
from feynml.interface import qgraf as iqgraf


def generate_diagrams(
    path: str,
    inparticles: List[str],
    outparticles: List[str],
    loops: int = 0,
    inmomenta: List[str] = None,
    outmomenta: List[str] = None,
    loop_momentum: str = "l",
    filter = None
) -> FeynML:
    """Generate FeynML diagrams from a UFO model."""
    if inmomenta is None:
        inmomenta = [f"p{i}" for i, p in enumerate(inparticles)]
    if outmomenta is None:
        outmomenta = [f"p{i+len(inparticles)}" for i, p in enumerate(outparticles)]
    # sanity check
    assert len(inmomenta) == len(inparticles)
    assert len(outmomenta) == len(outparticles)
    # Dress particles with momentum
    # TODO handle different names for particles
    inp = [f"{a}[{b}]" for a, b in zip(inparticles, inmomenta)]
    outp = [f"{a}[{b}]" for a, b in zip(outparticles, outmomenta)]
    # Load the UFO
    fm = load_ufo_model(path)
    # convert to feynmodel
    qgrafmodel = feynmodel_to_qgraf(fm, True, False)
    #print(qgrafmodel)
    if filter is not None:
        nq = ""
        # loop lines in the model
        for l in qgrafmodel.splitlines():
            hit = False
            for f in filter:
                # if the line is in the filter
                if f in l:
                    # add it to the new model
                    hit = True
            if not hit:
                nq += l + "\n"
        # overwrite the model
        qgrafmodel = nq
    #print(qgrafmodel)

    # Run pyQGRAF
    xml_string = qgraf.run(
        ",".join(inp),
        ",".join(outp),
        loops=loops,
        loop_momentum=loop_momentum,
        model=qgrafmodel,
        style=iqgraf.get_style(),
    )

    # Load FeynML
    parser = XmlParser()
    fml = parser.from_string(xml_string, FeynML)
    # Return list of diagrams
    return fml
